////////////////////////////////////////////////////////////////////////////////
//
//  IDEUM
//  Copyright 2011-2015 Ideum
//  All Rights Reserved.
//
//  GestureWorks
//
//  File:    HelloSenser.cpp
//  Authors:  Ideum
//             
//  NOTICE: Ideum permits you to use, modify, and distribute this file only
//  in accordance with the terms of the license agreement accompanying it.
//
////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "ofMain.h"
#include "HelloSensor.h"

using namespace GWCTutorials;

const std::string HelloSensor::window_title("Hello Senser");

CameraListener leap_listener;
//Thread function necessary to continuously listen for LEAP events
DWORD LEAP_thread(LPVOID lpdwThreadParam) {
	Leap::Controller leap_controller(leap_listener);
	while(1) {}
	return 0;
}

HelloSensor::HelloSensor()
: last_tick(0)
{}

//--------------------------------------------------------------
void HelloSensor::setup(){
	if (!GestureWorks::Instance()->LoadGestureWorks(L"C:\\trunk\\GestureWorksCore\\refactor\\GestureWorksCore\\Debug\\GestureWorksCore32.dll")){
		MessageBox(NULL, L"Unable to load GestureWorksCore DLL", L"Error", 0);
		return;
	}

	// Initialize with (0,0) so GestureWorks will automatically detect our screen resolution
	GestureWorks::Instance()->InitializeGestureWorks(0, 0);

	// Register our window with GestureWorks for touch input. We could also do this with an HWND.
	// Alternatively, we could use some other method of getting touch input and feed it to GestureWorks via
	// the addEvent method.
	if (!GestureWorks::Instance()->RegisterWindowForTouchByName(std::wstring(window_title.begin(), window_title.end()))){
		MessageBox(NULL, L"Unable to register window for touch", L"Error", 0);
		return;
	}

	if (!GestureWorks::Instance()->LoadGML(L"C:\\trunk\\GestureWorksCore\\refactor\\GestureWorksCore\\basic_manipulation.gml")){
		MessageBox(NULL,L"Unable to load GML", L"Error", 0);
		return;
	}

	ofSetFullscreen(true);

	GestureWorks::Instance()->SetUsePixels(true);

	CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)&LEAP_thread, 0, 0, 0);
}

//The update function gets called once per cycle prior to the draw function
void HelloSensor::update(){

	// For consistency, we try to run GestureWorks at a constant rate - about 60Hz in this case
	clock_t this_tick(clock());
	if (this_tick - last_tick < 16) return;
	last_tick = this_tick;

	GestureWorks::Instance()->ProcessFrame();

	std::vector<gwc::PointEvent> point_events = GestureWorks::Instance()->ConsumePointEvents();
	handlePointEvents(point_events);
}

//This function is the last to get called every cycle, and is responsible
//for generating all screen artifacts
void HelloSensor::draw(){

	for (const auto& point : active_points){
		point.second.Draw();
	}
}

void HelloSensor::handlePointEvents(const std::vector<gwc::PointEvent>& point_events){
	for (const auto& point : point_events){
		std::pair<int, int> window_coords(normScreenToWindowPx(point.position.x, point.position.y, true));

		switch (point.status) {
		case gwc::TOUCHADDED:
		case gwc::TOUCHUPDATE:
			//If the point is being added, this will place it in our point map; the same line of code will update the point's
			//position if it's already present, so we can use this command to handle new points as well as point updates
			active_points[point.point_id] = TouchPoint(point.point_id, window_coords.first, window_coords.second);
			break;
		case gwc::TOUCHREMOVED:
			//Remove the point from the map
			active_points.erase(point.point_id);
			break;
		}
	}
}
