#pragma once

#include "ofMain.h"

namespace GWCTutorials{
	
	// Visual representation of a GestureWorks touchpoint
	class TouchPoint{
	public:
		TouchPoint();
		TouchPoint(long id, int x, int y);
		void SetPosition(int x, int y);
		void SetColor(unsigned int color);
		void SetRadius(int radius);
		void Draw() const;

	private:
		long id;
		int x;
		int y;
		int radius;
		unsigned int color;
	};

	// Visual representation of a rectangle that we can manipulate via gestures
	class TouchRectangle{
	public:
		TouchRectangle();
		TouchRectangle(const ofImage& image, int width);

		void SetPosition(int x, int y);
		void IncrementPosition(int dx, int dy);

		void SetScale(float scale);
		void IncrementScale(float ds);

		void Rotate(float dtheta, float rotation_center_x, float rotation_center_y);

		void Draw();
		bool Hit(int x, int y) const;

		bool AddPoint(long id);
		bool RemovePoint(long id);

	private:
		std::vector<long> owned_points;
		ofImage image;
		int width;
		float rotation;
		float scale;
		int x;
		int y;
	};

	// Converts a pair of normalized screen coordinates (between 0 and 1) to pixel values localized to the window.
	std::pair<int, int> normScreenToWindowPx(float screen_x, float screen_y, bool use_pixels);

	// Rotate a point about another point by a certain angle. We use this to realize more "natural" object rotations. 
	std::pair<int, int> rotateAboutPoint(int point_x, int point_y, int rotation_center_x, int rotation_center_y, float angle_in_rads);
}